"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IrcProvisioningError = exports.IrcErrCode = exports.RoomIdValidator = exports.UnlinkValidator = exports.QueryLinkValidator = exports.LinkValidator = void 0;
const matrix_appservice_bridge_1 = require("matrix-appservice-bridge");
const matrixRoomIdValidation = {
    type: "string",
    pattern: "^!.*:.*$"
};
const validationProperties = {
    matrix_room_id: matrixRoomIdValidation,
    remote_room_channel: {
        type: "string",
        pattern: "^([#+&]|(![A-Z0-9]{5}))[^\\s:,]+$"
    },
    remote_room_server: {
        type: "string",
        pattern: "^[a-z\\.0-9:-]+$"
    },
    op_nick: {
        type: "string"
    },
    key: {
        type: "string"
    }
};
exports.LinkValidator = new matrix_appservice_bridge_1.ConfigValidator({
    type: "object",
    properties: validationProperties,
    required: [
        "matrix_room_id",
        "remote_room_channel",
        "remote_room_server",
        "op_nick",
    ]
});
exports.QueryLinkValidator = new matrix_appservice_bridge_1.ConfigValidator({
    type: "object",
    properties: validationProperties,
    required: [
        "remote_room_channel",
        "remote_room_server",
        "key"
    ]
});
exports.UnlinkValidator = new matrix_appservice_bridge_1.ConfigValidator({
    type: "object",
    properties: validationProperties,
    required: [
        "matrix_room_id",
        "remote_room_channel",
        "remote_room_server",
    ]
});
exports.RoomIdValidator = new matrix_appservice_bridge_1.ConfigValidator({
    type: "object",
    properties: {
        "matrix_room_id": matrixRoomIdValidation,
    },
    required: [
        "matrix_room_id",
    ],
});
var IrcErrCode;
(function (IrcErrCode) {
    IrcErrCode["UnknownNetwork"] = "IRC_UNKNOWN_NETWORK";
    IrcErrCode["UnknownChannel"] = "IRC_UNKNOWN_CHANNEL";
    IrcErrCode["UnknownRoom"] = "IRC_UNKNOWN_ROOM";
    IrcErrCode["DoubleBridge"] = "IRC_DOUBLE_BRIDGE";
    IrcErrCode["ExistingMapping"] = "IRC_EXISTING_MAPPING";
    IrcErrCode["ExistingRequest"] = "IRC_EXISTING_REQUEST";
    IrcErrCode["NotEnoughPower"] = "IRC_NOT_ENOUGH_POWER";
    IrcErrCode["BadOpTarget"] = "IRC_BAD_OPERATOR_TARGET";
    IrcErrCode["BridgeAtLimit"] = "IRC_BRIDGE_AT_LIMIT";
})(IrcErrCode = exports.IrcErrCode || (exports.IrcErrCode = {}));
const ErrCodeToStatusCode = {
    IRC_UNKNOWN_NETWORK: 404,
    IRC_UNKNOWN_CHANNEL: 404,
    IRC_UNKNOWN_ROOM: 404,
    IRC_EXISTING_MAPPING: 409,
    IRC_EXISTING_REQUEST: 409,
    IRC_DOUBLE_BRIDGE: 409,
    IRC_NOT_ENOUGH_POWER: 403,
    IRC_BAD_OPERATOR_TARGET: 400,
    IRC_BRIDGE_AT_LIMIT: 500
};
class IrcProvisioningError extends Error {
    constructor(error, errcode, statusCode = -1, additionalContent = {}) {
        super(`API error ${errcode}: ${error}`);
        this.error = error;
        this.errcode = errcode;
        this.statusCode = statusCode;
        this.additionalContent = additionalContent;
        if (statusCode === -1) {
            this.statusCode = ErrCodeToStatusCode[errcode];
        }
    }
    get jsonBody() {
        return Object.assign({ errcode: this.errcode, error: this.error }, this.additionalContent);
    }
    apply(response) {
        response.status(this.statusCode).send(this.jsonBody);
    }
}
exports.IrcProvisioningError = IrcProvisioningError;
//# sourceMappingURL=Schema.js.map